@@ -27,7 +27,7 @@
  * list of people on the gedit Team.
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-commands-file.c 6498 2008-09-07 13:50:57Z pborelli $
+ * $Id: gedit-commands-file.c 6912 2009-02-22 17:00:21Z pborelli $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -292,7 +292,12 @@ load_uri_list (GeditWindow         *window,
 
 	for (u = uris; u != NULL; u = u->next)
 	{
-		files = g_slist_prepend (files, g_file_new_for_uri (u->data));
+		gchar *uri = u->data;
+
+		if (gedit_utils_is_valid_uri (uri))
+			files = g_slist_prepend (files, g_file_new_for_uri (uri));
+		else
+			g_warning ("invalid uri: %s", uri);
 	}
 	files = g_slist_reverse (files);
 
@@ -356,13 +361,13 @@ gedit_commands_load_uris (GeditWindow         *window,
  */
 gint
 _gedit_cmd_load_files_from_prompt (GeditWindow         *window,
-				   const GSList        *uris,
+				   GSList              *files,
 				   const GeditEncoding *encoding,
 				   gint                 line_pos)
 {
 	gedit_debug (DEBUG_COMMANDS);
 
-	return load_uri_list (window, uris, encoding, line_pos, TRUE);
+	return load_file_list (window, files, encoding, line_pos, TRUE);
 }
 
 static void
@@ -956,6 +961,20 @@ _gedit_cmd_file_save_as (GtkAction   *action,
 	file_save_as (tab, window);
 }
 
+static gboolean
+document_needs_saving (GeditDocument *doc)
+{
+	if (gtk_text_buffer_get_modified (GTK_TEXT_BUFFER (doc)))
+		return TRUE;
+
+	/* we check if it was deleted only for local files
+	 * since for remote files it may hang */
+	if (gedit_document_is_local (doc) && gedit_document_get_deleted (doc))
+		return TRUE;
+
+	return FALSE;
+}
+
 /*
  * The docs in the list must belong to the same GeditWindow.
  */
@@ -998,9 +1017,7 @@ _gedit_cmd_file_save_documents_list (GeditWindow *window,
 			if (gedit_document_is_untitled (doc) || 
 			    gedit_document_get_readonly (doc))
 			{
-				/* Do not save unmodified utitled or readonly documents */
-				if (gtk_text_buffer_get_modified (GTK_TEXT_BUFFER (doc)) ||
-			     	    gedit_document_get_deleted (doc))
+				if (document_needs_saving (doc))
 			     	{
 				     	tabs_to_save_as = g_slist_prepend (tabs_to_save_as,
 									   t);
@@ -1350,8 +1367,7 @@ tab_state_changed_while_saving (GeditTab    *tab,
 
 		/* If the saving operation failed or was interrupted, then the
 		   document is still "modified" -> do not close the tab */
-		if (gtk_text_buffer_get_modified (GTK_TEXT_BUFFER (doc)) ||
-		    gedit_document_get_deleted (doc))
+		if (document_needs_saving (doc))
 			return;
 
 		/* Close the document only if it has been succesfully saved.
@@ -1474,8 +1490,7 @@ save_and_close_all_documents (const GList  *docs,
 			    (state != GEDIT_TAB_STATE_REVERTING)) /* CHECK: is this the right behavior with REVERTING ?*/
 			{			
 				/* The document must be saved before closing */
-				g_return_if_fail (gtk_text_buffer_get_modified (GTK_TEXT_BUFFER (doc)) ||
-		     				  gedit_document_get_deleted (doc));
+				g_return_if_fail (document_needs_saving (doc));
 				
 				/* FIXME: manage the case of local readonly files owned by the
 				   user is running gedit - Paolo (Dec. 8, 2005) */
