@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-tab.h 5428 2007-01-08 22:02:42Z pborelli $
+ * $Id: gedit-tab.h 6116 2008-01-24 22:15:19Z pborelli $
  */
 
 #ifndef __GEDIT_TAB_H__
@@ -35,7 +35,6 @@
 
 #include <gedit/gedit-view.h>
 #include <gedit/gedit-document.h>
-#include <gedit/gedit-print.h>
 
 G_BEGIN_DECLS
 
@@ -63,7 +62,6 @@ typedef enum
  */
 #define GEDIT_TYPE_TAB              (gedit_tab_get_type())
 #define GEDIT_TAB(obj)              (G_TYPE_CHECK_INSTANCE_CAST((obj), GEDIT_TYPE_TAB, GeditTab))
-#define GEDIT_TAB_CONST(obj)        (G_TYPE_CHECK_INSTANCE_CAST((obj), GEDIT_TYPE_TAB, GeditTab const))
 #define GEDIT_TAB_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST((klass), GEDIT_TYPE_TAB, GeditTabClass))
 #define GEDIT_IS_TAB(obj)           (G_TYPE_CHECK_INSTANCE_TYPE((obj), GEDIT_TYPE_TAB))
 #define GEDIT_IS_TAB_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), GEDIT_TYPE_TAB))
@@ -147,14 +145,11 @@ void		 _gedit_tab_save		(GeditTab            *tab);
 void		 _gedit_tab_save_as		(GeditTab            *tab,
 						 const gchar         *uri,
 						 const GeditEncoding *encoding);
-void		 _gedit_tab_print		(GeditTab            *tab,
-						 GeditPrintJob       *pjob,
-						 GtkTextIter         *start, 
-			  			 GtkTextIter         *end);
-void		 _gedit_tab_print_preview	(GeditTab            *tab,
-						 GeditPrintJob       *pjob,
-						 GtkTextIter         *start, 
-			  			 GtkTextIter         *end);
+
+void		 _gedit_tab_page_setup		(GeditTab            *tab);
+void		 _gedit_tab_print		(GeditTab            *tab);
+void		 _gedit_tab_print_preview	(GeditTab            *tab);
+
 void		 _gedit_tab_mark_for_closing	(GeditTab	     *tab);
 
 gboolean	 _gedit_tab_can_close		(GeditTab	     *tab);
