@@ -27,7 +27,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-utils.h 6727 2008-12-25 21:02:48Z pborelli $
+ * $Id: gedit-utils.h 6926 2009-02-28 13:03:07Z pborelli $
  */
 
 #ifndef __GEDIT_UTILS_H__
@@ -41,8 +41,8 @@
 G_BEGIN_DECLS
 
 /* useful macro */
-#define GBOOLEAN_TO_POINTER(i) ((gpointer) ((i) ? 2 : 1))
-#define GPOINTER_TO_BOOLEAN(i) ((gboolean) ((((gint)(i)) == 2) ? TRUE : FALSE))
+#define GBOOLEAN_TO_POINTER(i) (GINT_TO_POINTER ((i) ? 2 : 1))
+#define GPOINTER_TO_BOOLEAN(i) ((gboolean) ((GPOINTER_TO_INT(i) == 2) ? TRUE : FALSE))
 
 #define IS_VALID_BOOLEAN(v) (((v == TRUE) || (v == FALSE)) ? TRUE : FALSE)
 
@@ -111,6 +111,9 @@ gchar		*gedit_utils_make_valid_utf8		(const char *name);
 /* Note that this function replace home dir with ~ */
 gchar		*gedit_utils_uri_get_dirname		(const char *uri);
 
+gchar		*gedit_utils_location_get_dirname_for_display
+							(GFile *location);
+
 gchar		*gedit_utils_replace_home_dir_with_tilde (const gchar *uri);
 
 guint		 gedit_utils_get_current_workspace	(GdkScreen *screen);
